GridDrawer = class()

function GridDrawer:init(scene)
    self.scene = scene
    self.origin = vec3(0, 0, 0)
    
    function updatePositionText()
        XYZ_Position = tostring(X_Position)..", "..
        tostring(Y_Position)..", "..tostring(Z_Position)
    end
    -- Parameters for grid control
    parameter.integer("NumLines", 1, 50, 24)
    parameter.integer("X_Position", -50, 50, 0, function(value) 
        self.origin.x = value 
        updatePositionText()
    end)
    parameter.integer("Y_Position", -50, 50, 0, function(value) 
        self.origin.y = value 
        updatePositionText()
    end)
    parameter.integer("Z_Position", -50, 50, 0, function(value) 
        self.origin.z = value 
        updatePositionText()
    end)
    
    parameter.text("XYZ_Position", "0,0,0")
    
    parameter.action("apply coordinates", function()
        local x, y, z = XYZ_Position:match("([^,]+),([^,]+),([^,]+)")
        local nX, nY, nZ = tonumber(x), tonumber(y), tonumber(z)
        if (type(nX) == "number") and 
        (type(nY) == "number") and 
        (type(nZ) == "number") then
            self.origin = vec3(nX, nY, nZ)
            -- Update sliders
            X_Position = self.origin.x
            Y_Position = self.origin.y
            Z_Position = self.origin.z
        end
    end)
    
    -- Color chooser
    parameter.color("chosenColor", 255, 0, 0, function(value)
        local col1, col2, col3 = getTriangleColors(value)
        self.colors = {x = col1, y = col2, z = col3}
    end)
    local col1, col2, col3 = getTriangleColors(color(255, 0, 0))
    self.colors = {x = col1, y = col2, z = col3}
end

function GridDrawer:draw()
    self:drawGridPlane(self.origin, "x", NumLines, self.colors.x)
    self:drawGridPlane(self.origin, "y", NumLines, self.colors.y)
    self:drawGridPlane(self.origin, "z", NumLines, self.colors.z)
    self:drawOriginText()
    self:drawAxisLabel("x", self.colors.x, self.origin.x)
    self:drawAxisLabel("y", self.colors.y, self.origin.y)
    self:drawAxisLabel("z", self.colors.z, self.origin.z)
end

function GridDrawer:drawAxisLabel(axis, color, coordinate)
    local offset = NumLines * 0.5
    local pos
    local formattedCoord = string.format("%.0f", coordinate)
    local labelString = "plane:\n"..axis..formattedCoord
    
    if axis == "x" then
        pos = self.origin + vec3(0, offset, offset)
    elseif axis == "y" then
        pos = self.origin + vec3(-offset, 0, offset)
    elseif axis == "z" then
        pos = self.origin + vec3(-offset, offset, 0)
    end
    
    local camera = self.scene.camera:get(craft.camera)
    local screenPos = camera:worldToScreen(pos)
    
    -- Calculate the distance from the camera to the position
    local distance = pos:dist(self.scene.camera.position)
    
    -- Adjust the font size based on the distance
    local aFontSize = math.max(110 / (distance * 0.1), 15)
    
    pushStyle()
    pushMatrix()
    resetMatrix()
    viewMatrix(matrix())
    ortho()
    fill(color)
    fontSize(aFontSize)
    textAlign(CENTER)
    text(labelString, screenPos.x, screenPos.y + (aFontSize * 1.5))
    popMatrix()
    popStyle()
end

function GridDrawer:drawOriginText()
    local camera = self.scene.camera:get(craft.camera)
    local screenPos = camera:worldToScreen(self.origin)
    
    -- Calculate the distance from the camera to the origin
    local distance = self.origin:dist(self.scene.camera.position)
    
    -- Adjust the font size based on the distance
    local aFontSize = math.max(80 / (distance * 0.1), 15)
    
    -- Format the origin's coordinates to remove the decimal when it's 0
    local formattedX = string.format("%.0f", self.origin.x)
    local formattedY = string.format("%.0f", self.origin.y)
    local formattedZ = string.format("%.0f", self.origin.z)
    
    -- Draw the text slightly above the origin's screen position
    pushMatrix()
    resetMatrix()
    viewMatrix(matrix())
    ortho()
    fill(206)
    fontSize(aFontSize)
    text(string.format("(%s, %s, %s)", formattedX, formattedY, formattedZ), screenPos.x, screenPos.y + 20)
    popMatrix()
end

function GridDrawer:touched(touch)
    local camera = self.scene.camera:get(craft.camera)
    local screenPos = camera:worldToScreen(self.origin)
    
    -- Calculate the distance between the touch's start position and the origin's screen position
    local distanceToOrigin = vec2(touch.x, touch.y):dist(vec2(screenPos.x, screenPos.y))
    
    -- Only allow dragging if the touch started close to the origin (e.g., within 50 pixels)
    if touch.state == BEGAN and distanceToOrigin <= 50 then
        self.allowDrag = true
        -- Calculate the distance from the camera to the origin in world space
        self.cameraToOriginDistance = self.origin:dist(self.scene.camera.position)
    elseif touch.state == ENDED or touch.state == CANCELLED then
        self.allowDrag = false
    end
    
    if self.allowDrag and touch.state == MOVING then
        -- Convert the screen touch position to world position using the distance from the camera to the origin
        local worldPos = camera:screenToWorld(vec3(touch.x, touch.y, self.cameraToOriginDistance))
        
        -- Round the world position values to the nearest whole number
        worldPos.x = math.floor(worldPos.x + 0.5)
        worldPos.y = math.floor(worldPos.y + 0.5)
        worldPos.z = math.floor(worldPos.z + 0.5)
        
        -- Update the origin to the new world position
        self.origin = worldPos
        
        -- Update the sliders and text field
        X_Position = self.origin.x
        Y_Position = self.origin.y
        Z_Position = self.origin.z
        updatePositionText()
    end
end

function GridDrawer:drawGridPlane(origin, axis, numLines, color)
    local step = 1 -- distance between lines
    local halfExtent = step * numLines * 0.5
    local offset = 0.02 -- The amount to offset the additional lines for "thickness"
    
    local function drawThickerLine(startPos, endPos, color)
        self.scene.debug:line(startPos, endPos, color)
        self.scene.debug:line(startPos + vec3(offset, offset, offset), endPos + vec3(offset, offset, offset), color)
        self.scene.debug:line(startPos - vec3(offset, offset, offset), endPos - vec3(offset, offset, offset), color)
    end
    
    local start = -halfExtent
    local finish = halfExtent
    if numLines % 2 == 1 then
        start = start + 0.5
        finish = finish - 0.5
    end
    
    for i = start, finish, step do
        local isThickLine = (i % 10 == 0)
        if isThickLine then
            -- Draw thicker lines every 10 units
            if axis == "x" then
                drawThickerLine(vec3(origin.x, origin.y + i, origin.z - halfExtent), vec3(origin.x, origin.y + i, origin.z + halfExtent), color)
                drawThickerLine(vec3(origin.x, origin.y - halfExtent, origin.z + i), vec3(origin.x, origin.y + halfExtent, origin.z + i), color)
            elseif axis == "y" then
                drawThickerLine(vec3(origin.x + i, origin.y, origin.z - halfExtent), vec3(origin.x + i, origin.y, origin.z + halfExtent), color)
                drawThickerLine(vec3(origin.x - halfExtent, origin.y, origin.z + i), vec3(origin.x + halfExtent, origin.y, origin.z + i), color)
            elseif axis == "z" then
                drawThickerLine(vec3(origin.x + i, origin.y - halfExtent, origin.z), vec3(origin.x + i, origin.y + halfExtent, origin.z), color)
                drawThickerLine(vec3(origin.x - halfExtent, origin.y + i, origin.z), vec3(origin.x + halfExtent, origin.y + i, origin.z), color)
            end
        else
            -- Draw regular lines
            if axis == "x" then
                self.scene.debug:line(vec3(origin.x, origin.y + i, origin.z - halfExtent), vec3(origin.x, origin.y + i, origin.z + halfExtent), color)
                self.scene.debug:line(vec3(origin.x, origin.y - halfExtent, origin.z + i), vec3(origin.x, origin.y + halfExtent, origin.z + i), color)
            elseif axis == "y" then
                self.scene.debug:line(vec3(origin.x + i, origin.y, origin.z - halfExtent), vec3(origin.x + i, origin.y, origin.z + halfExtent), color)
                self.scene.debug:line(vec3(origin.x - halfExtent, origin.y, origin.z + i), vec3(origin.x + halfExtent, origin.y, origin.z + i), color)
            elseif axis == "z" then
                self.scene.debug:line(vec3(origin.x + i, origin.y - halfExtent, origin.z), vec3(origin.x + i, origin.y + halfExtent, origin.z), color)
                self.scene.debug:line(vec3(origin.x - halfExtent, origin.y + i, origin.z), vec3(origin.x + halfExtent, origin.y + i, origin.z), color)
            end
        end
    end
end